// Squawk messenger. 
// Copyright (C) 2019  Yury Gubich <blue@macaw.me>
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef TEXTMETER_H
#define TEXTMETER_H

#include <list>
#include <set>

#include <QFontMetrics>
#include <QFont>
#include <QSize>
#include <QFontDatabase>

class TextMeter
{
public:
    TextMeter();
    ~TextMeter();
    void initializeFonts(const QFont& font);
    QSize boundingSize(const QString& text, const QSize& limits) const;

private:
    QFontDatabase base;
    std::list<QFontMetrics> fonts;

    struct Helper {
        Helper(int maxWidth);
        int width;
        int height;
        int wordWidth;
        int wordHeight;
        int delayedSpaceWidth;
        int maxWidth;
        bool wordBeganWithTheLine;
        QString text;
        QString word;
        QChar lastSpace;
        QSize size;

        void computeNewLine(const QFontMetrics& metrics, int horizontalAdvance, const QChar& ch);
        void computeChar(const QChar& ch, const QFontMetrics& metrics);
        void computeNewWord();
        void beginNewLine();
        bool lineOverflow(const QFontMetrics& metrics, int horizontalAdvance, const QChar& ch);
    };

    static bool newLine(const QChar& ch);
    static bool newWord(const QChar& ch);
    static bool visible(const QChar& ch);
    static bool printOnLineBreak(const QChar& ch);

};

#endif // TEXTMETER_H
